package za.co.symbiotics.test;

import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.TextMessage;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Install;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.log.Log;
import org.jboss.seam.mock.SeamTest;
import org.testng.Assert;
import org.testng.annotations.Test;

import za.co.symbiotics.seam.test.entity.TestEntity;
import za.co.symbiotics.test.handler.TopicHandler;

public class SeamErrorTest extends SeamTest {
    @Test
    public void testGenerateEvent() throws Exception {
        new FacesRequest() {
            @Override
            protected void invokeApplication() throws Exception {
                Assert.assertNotNull(getValue("#{entityManager}"), "entityManager not found");
                TestFeedSender sender = (TestFeedSender) getValue("#{testFeedSender}");
                Assert.assertNotNull(sender, "testFeedSender not found");
                MockTopicHandler topicHandler = (MockTopicHandler) getValue("#{topicHandler}");
                Assert.assertNotNull(topicHandler, "topicHandler not found");
                invokeAction("#{testFeedSender.send}");
            }
        }.run();
    }

    @Test(dependsOnMethods = "testGenerateEvent")
    public void testReceiveEvent() throws Exception {
        new FacesRequest() {
            @Override
            protected void invokeApplication() throws Exception {
                TestFeedSender sender = (TestFeedSender) getValue("#{testFeedSender}");
                Assert.assertNotNull(sender, "testFeedSender not found");
                MockTopicHandler topicHandler = (MockTopicHandler) getValue("#{topicHandler}");
                Assert.assertNotNull(topicHandler, "topicHandler not found");
                topicHandler.waitForEvent(60000);
                Assert.assertTrue(topicHandler.isReceived(), "Did not receive event before timeout");
                Thread.sleep(1000);
                Assert.assertTrue(topicHandler.isValid(), "Did not receive valid event");
            }
        }.run();
    }

    @Name("topicHandler")
    @Install(precedence = Install.MOCK)
    @Scope(ScopeType.APPLICATION)
    static public class MockTopicHandler implements TopicHandler {
        @Logger
        Log logger;

        private boolean received = false;

        private boolean valid = false;

        public void handleEvent(Object obj) {
            logger.info("Received Event:" + obj);
            setReceived(true);
            Assert.assertNotNull(obj, "Event is null");
            Assert.assertTrue(obj instanceof TestEntity, "event not instanceof TestEntity");
            TestEntity entity = (TestEntity) obj;
            Assert.assertEquals(entity.getValue1(), "test value 1");
            Assert.assertEquals(entity.getValue2(), "test value 2");
            valid = true;
            logger.info("Processed Event");
        }

        public synchronized void waitForEvent(long timeout) throws InterruptedException {
            if (!received) {
                logger.info("waiting for event");
                wait(timeout);
            }
            if (received) {
                logger.info("event received");
            }
        }

        public boolean isReceived() {
            return received;
        }

        public void setReceived(boolean received) {
            this.received = received;
            synchronized (this) {
                notifyAll();
            }
        }

        public boolean isValid() {
            return valid;
        }

        public void setValid(boolean valid) {
            this.valid = valid;
        }
    }

    @Name("testFeedSender")
    public static class TestFeedSender {
        @In
        private QueueSender feedQueueSender;

        @In
        private QueueSession queueSession;

        @Logger
        Log logger;

        public void send() throws JMSException {
            logger.info("Sending Feed Message");
            Assert.assertNotNull(feedQueueSender, "feedQueueSender is null");
            Assert.assertNotNull(queueSession, "queueSession is null");
            TextMessage textMessage = queueSession.createTextMessage("TestMessage");
            textMessage.setStringProperty("value1", "test value 1");
            textMessage.setStringProperty("value2", "test value 2");
            feedQueueSender.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
            feedQueueSender.send(textMessage);
            logger.info("Sent Feed Message");
        }

        public QueueSender getFeedQueueSender() {
            return feedQueueSender;
        }

        public void setFeedQueueSender(QueueSender feedQueueSender) {
            this.feedQueueSender = feedQueueSender;
        }

        public QueueSession getQueueSession() {
            return queueSession;
        }

        public void setQueueSession(QueueSession queueSession) {
            this.queueSession = queueSession;
        }
    }

}
